`include "phy_const.v"

///////////////////////////////////////////////////////////
//Bytes come M.S.B. first, but the nibbles with in a byte
//come L.S.Nibble first. Use this module to reverse the
//nibble order.
//Note: be aware that the last nibble within this "reordering"
// shift register is not valid until both nibble of a byte
// has been shifted in.
///////////////////////////////////////////////////////////
module rx_shift(I, O, C, R);
   input [3:0]   I;
   output [63:0] O;
   input 	 C, R;

   reg [63:0] 	 O;   
   reg [3:0] 	 tmp;
   reg 		 lowerNibble;

   always @(posedge C)
     if (R) begin
	lowerNibble <= 1;
	O <= 0;
	tmp <= 0;	
     end
     else if (lowerNibble) begin
	{O, tmp} <= {O[59:0], tmp, I};
	lowerNibble <= 0;
     end
     else begin
	O <= {O[59:4] ,I ,tmp};
	lowerNibble <= 1;	
     end
endmodule // rx_shift

//////////////////////////////////////////////////////////
//the Receiver module
//////////////////////////////////////////////////////////
module receiver (RX_DV, RX_ER, RX_CLK, RX_D,
		 CHANNEL, MSG, RESET, DEBUG);
   
   input RX_DV, RX_ER, RX_CLK;
   input [3:0] RX_D;
   input [7:0] CHANNEL;
   output [63:0] MSG;
   input 	 RESET;
   inout [31:0]  DEBUG;

   //////////////////////////////////////////////////
   //wire and reg declarations...




   
   
   ////////////////////////////////////////////////////
   //packet cycle timing parameter.
   //enter parameters for the important cycle counts of
   //the nibble counter
   parameter 	 cycleMacType=,
		 cycleRtpType=,
		 cycleSSRC=,
		 cycleRstAccum=,		 
		 cycleEndOfPayLoad=,
 		 accumPeriodOffset=;


   /////////////////////////////////////////////////////
   //Dataflow assignments
   //example:
   // assign invalidMacType = (cycleMacType == nibbleCounterData) &
   //                         (shiftRegData[15:0] != `eTvMacType);
   
 
   
   /////////////////////////////////////////////////////
   //Datapath:
   //
   //write the code for the acummulator and
   //Connect up the varius peices of the datapath
   //Hint:
   // not all the ports on all the modules are useful
   
   rx_shift shft_reg(.I(),
		     .O(),
		     .C(),
		     .R());

   register #(64) LED_reg(.I(),
			  .O(),
			  .E(),
			  .R(),
			  .C());

   register #(64) result_reg(.I(),
			     .O(),
			     .E(),
			     .R(),
			     .C());

   //////////////////
   //accumulator:


   

   /////////////////
   
   counter #(11) nibbleCounter (.OUT(),
				.CLK(),
				.ENABLE(),
				.RESET());
   
   eth_crc crc_inst(.Clk(),
		    .Reset(),
		    //don't change the .Data port connection
		    .Data({RX_D[0],RX_D[1],RX_D[2],RX_D[3]}),
		    .Enable(),
		    .Crc(),
		    .CrcError());


		    
   /////////////////////////////////////////////////////
   //FSM:
   ////////////////////////////////////////////////////
   //state parameters:
   //example:		    
   //  parameter [1:0] idle=,
   //  		       receiving=,
   //                  receivingInvalid=;


   //DO NOT CHANGE THIS!!!!
   reg [1:0] 	 CS, NS;
   always @ (posedge RX_CLK)
     if (RESET)
       CS <= idle;
     else
       CS <= NS;
   
   ///////////////////////////////////////////////
   //next state and output logic
   always @ (/*fill in sensitiviy list*/) begin
      ///////////////////////////////////////////
      //default FSM output:
      //Make sure you assign all output of the
      //statemachine with a default value

		    
      /////////////////////////////////////////////
      //NS and output logic:
      //example:
      //  case (CS) /* synthesis full_case */
      //    idle: begin
      //      ...
      //    end
      //    receiving: begin
      //      ...
      //    end
      //    receivingInvalid: begin
      //      ...
      //    end
      //endcase


   end
endmodule